package ellightemu;

import com.sonycsl.echo.Echo;
import com.sonycsl.echo.eoj.device.DeviceObject;
import com.sonycsl.echo.processing.defaults.DefaultNodeProfile;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.scene.Scene;
import javafx.scene.effect.ColorAdjust;
import javafx.scene.effect.DropShadow;
import javafx.scene.effect.Effect;
import javafx.scene.effect.ImageInput;
import javafx.scene.effect.Reflection;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.StackPane;
import javafx.stage.Stage;

public class EllightEmu extends Application {
    public static final int port = 9999;

    private ImageView bg;
    private Effect stand, stand_d;
    private ServerSocket serversocket;
    private Service<String> service;

    @Override
    public void start(Stage primaryStage) throws IOException {
        StackPane root = new StackPane();
        Scene scene = new Scene(root, 300, 250);
        bg = new ImageView("ellightemu/stand.jpg");
        root.getChildren().add(bg);

        primaryStage.setTitle("ECHONET Lite General Lighting");
        primaryStage.setScene(scene);
        primaryStage.show();

        ImageInput imageinput;
        Reflection reflection;
        ColorAdjust coloradjust;
        DropShadow dropshadow;

        imageinput = new ImageInput(new Image("ellightemu/stand.jpg"));
        reflection = new Reflection();
        reflection.setInput(imageinput);
        dropshadow = new DropShadow();
        dropshadow.setInput(reflection);
        stand = dropshadow;
        coloradjust = new ColorAdjust();
        coloradjust.setBrightness(-0.75);
        coloradjust.setInput(imageinput);
        dropshadow = new DropShadow();
        dropshadow.setInput(coloradjust);
        stand_d = dropshadow;
        bg.setEffect(stand_d);

        try {
            serversocket = new ServerSocket(port);
            serversocket.setReuseAddress(true);
        } catch (IOException ex) {
            Logger.getLogger(EllightEmu.class.getName()).log(Level.SEVERE,
                             null, ex);
        }
        service = new Service<String>() {
            @Override
            protected Task<String> createTask() {
                Task<String> task = new Task<String>() {
                    @Override
                    protected String call() throws IOException {
                        Socket socket = serversocket.accept();
                        BufferedReader reader =
                                new BufferedReader(new InputStreamReader
                                                   (socket.getInputStream()));
                        return reader.readLine();
                    }
                };
                task.setOnSucceeded((WorkerStateEvent event) -> {
                    String value = event.getSource().getValue().toString();
                    if (value.equalsIgnoreCase("1: on")) {
                        bg.setEffect(stand);
                    } else if (value.equalsIgnoreCase("1: off")) {
                        bg.setEffect(stand_d);
                    }
                    service.restart();
                });
                return task;
            }
        };
        service.start();

        Runtime rt = Runtime.getRuntime();
        Echo.start(new DefaultNodeProfile(),
                   new DeviceObject[]{new MyLighting()});
    }

    public static void main(String[] args) {
        launch(args);
    }
}
